#!/usr/bin/env python
# -*- coding: utf-8 -*-

#########################################################################
# Copyright/License Notice (BSD License)                                #
#########################################################################
#########################################################################
# Copyright (c) 2012, Daniel Knaggs                                     #
# All rights reserved.                                                  #
#                                                                       #
# Redistribution and use in source and binary forms, with or without    #
# modification, are permitted provided that the following conditions    #
# are met: -                                                            #
#                                                                       #
#   * Redistributions of source code must retain the above copyright    #
#     notice, this list of conditions and the following disclaimer.     #
#                                                                       #
#   * Redistributions in binary form must reproduce the above copyright #
#     notice, this list of conditions and the following disclaimer in   #
#     the documentation and/or other materials provided with the        #
#     distribution.                                                     #
#                                                                       #
#   * Neither the name of the author nor the names of its contributors  #
#     may be used to endorse or promote products derived from this      #
#     software without specific prior written permission.               #
#                                                                       #
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   #
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     #
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT  #
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, #
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT      #
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, #
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY #
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   #
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE #
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  #
#########################################################################


class DanZFS():
	def __init__(self, colour_logging = True, debug_mode = False):
		import subprocess
		import sys
		
		
		self.subprocess = subprocess
		self.sys = sys
		
		self.COLOUR_LOGGING = colour_logging
		self.DEBUG_MODE = debug_mode
		self.EXECUTABLE_ZFS = "zfs"
		self.EXECUTABLE_ZPOOL = "zpool"
		self.VERSION = "0.1.2"
		self.ZFS_GET = [self.EXECUTABLE_ZFS, "get", "all"]
		self.ZFS_LIST_FILESYSTEMS = [self.EXECUTABLE_ZFS, "list", "-t", "filesystem"]
		self.ZFS_LIST_SNAPSHOTS = [self.EXECUTABLE_ZFS, "list", "-t", "snapshot"]
		self.ZFS_LIST_VOLUMES = [self.EXECUTABLE_ZFS, "list", "-t", "volume"]
		self.ZPOOL_DISK_LINE_OFFSET = 2
		self.ZPOOL_KEYWORD_ACTION = "action:"
		self.ZPOOL_KEYWORD_CONFIG = "config:"
		self.ZPOOL_KEYWORD_ERRORS = "errors:"
		self.ZPOOL_KEYWORD_POOL = "pool:"
		self.ZPOOL_KEYWORD_SCAN = "scan:"
		self.ZPOOL_KEYWORD_SEE = "see:"
		self.ZPOOL_KEYWORD_STATE = "state:"
		self.ZPOOL_KEYWORD_STATUS = "status:"
		self.ZPOOL_KEYWORDS = [self.ZPOOL_KEYWORD_ACTION, self.ZPOOL_KEYWORD_CONFIG, self.ZPOOL_KEYWORD_ERRORS, self.ZPOOL_KEYWORD_POOL, self.ZPOOL_KEYWORD_SCAN, self.ZPOOL_KEYWORD_SEE, self.ZPOOL_KEYWORD_STATE, self.ZPOOL_KEYWORD_STATUS]
		self.ZPOOL_LIST = [self.EXECUTABLE_ZPOOL, "list"]
		self.ZPOOL_IOSTAT = [self.EXECUTABLE_ZPOOL, "iostat"]
		self.ZPOOL_STATUS = [self.EXECUTABLE_ZPOOL, "status"]
		
		
		self.log("__init__", "Information", "Displaying copyright/license...")
		
		print """
#########################################################################
# Copyright/License Notice (BSD License)                                #
#########################################################################
#########################################################################
# Copyright (c) 2012, Daniel Knaggs                                     #
# All rights reserved.                                                  #
#                                                                       #
# Redistribution and use in source and binary forms, with or without    #
# modification, are permitted provided that the following conditions    #
# are met: -                                                            #
#                                                                       #
#   * Redistributions of source code must retain the above copyright    #
#     notice, this list of conditions and the following disclaimer.     #
#                                                                       #
#   * Redistributions in binary form must reproduce the above copyright #
#     notice, this list of conditions and the following disclaimer in   #
#     the documentation and/or other materials provided with the        #
#     distribution.                                                     #
#                                                                       #
#   * Neither the name of the author nor the names of its contributors  #
#     may be used to endorse or promote products derived from this      #
#     software without specific prior written permission.               #
#                                                                       #
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS   #
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT     #
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT  #
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, #
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT      #
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, #
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY #
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   #
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE #
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.  #
#########################################################################
"""
		
		
		# Welcome
		self.log("__init__", "Information", "Welcome to DanZFS v%s." % self.VERSION)
		
		# Python version checking
		self.log("__init__", "Information", "Checking Python version...")
		
		pyv = self.sys.version_info
		self.log("__init__", "Information", "Running under Python v%d.%d.%d." % (pyv[0], pyv[1], pyv[2]))
	
	def dispose(self):
		if self.DEBUG_MODE:
			self.log("dispose", "Information", "Running...")
		
		
		self.subprocess = None
		self.sys = None
	
	def log(self, module, level, message):
		header = "DanZFS"
		
		if self.COLOUR_LOGGING:
			header = "\033[1;35m%s\033[1;m" % header
			
			
			# Colourise the module
			c1 = "\033[1;36m"
			c2 = "\033[1;m"
			
			module = c1 + module + c2
			
			
			# Colourise the level
			if level == "Information":
				c1 = "\033[1;32m"
				
			elif level == "Warning":
				c1 = "\033[1;33m"
				
			elif level == "Exception" or level == "Error":
				c1 = "\033[1;31m"
				
			else:
				c1 = "\033[1;34m"
			
			level = c1 + level + c2
			
			
			# Colourise the message
			c1 = "\033[1;37m"
			
			message = c1 + message + c2
		
		print "%s/%s()/%s - %s" % (header, module, level, message)
	
	def run(self, cmd):
		if self.DEBUG_MODE:
			self.log("run", "Information", "Running...")
		
		
		p = self.subprocess.Popen(cmd, stdout = self.subprocess.PIPE, stderr = self.subprocess.PIPE)
		stdout, stderr = p.communicate()
		
		p.wait()
		returncode = p.returncode
		p = None
		
		
		if self.DEBUG_MODE and returncode <> 0:
			self.log("run", "Warning", "Returned exit code %d." % returncode)
		
		return [stdout, stderr, returncode]
	
	def trimExcessiveSpaces(self, data):
		if self.DEBUG_MODE:
			self.log("trimExcessiveSpaces", "Information", "Running...")
		
		
		ret = str(data)
		
		while True:
			if ret.find("  ") <> -1:
				ret = ret.replace("  ", " ")
				
			else:
				break
		
		return ret
	
	def zfsGet(self, specific = ""):
		if self.DEBUG_MODE:
			self.log("zfsGet", "Information", "Running...")
		
		
		cmd = list(self.ZFS_GET)
		
		if specific <> "":
			cmd.append(specific)
		
		
		stdout, stderr, returncode = self.run(cmd)
		
		lines = stdout.split("\n")
		
		if len(lines) == 0:
			raise Exception("There doesn't appear to be anything returned.")
			
		else:
			if lines[0].startswith("no pools available"):
				raise Exception("No pools are available.")
				
			else:
				if self.DEBUG_MODE:
					for l in lines:
						self.log("zfsGet", "Debug", l)
				
				
				properties = []
				property = None
				
				num = 0
				
				
				# Clean up the data first
				cleaned = []
				
				for l in lines:
					l = self.trimExcessiveSpaces(l).replace("\t", "").strip()
					cleaned.append(l)
				
				lines = cleaned
				cleaned = None
				
				
				for l in lines:
					if not l.startswith("NAME PROPERTY VALUE SOURCE"):
						d = l.split(" ")
						
						if len(d) == 4:
							property = {"name": d[0], "property": d[1], "value": d[2], "source": d[3]}
							properties.append(property)
				
				
				if self.DEBUG_MODE:
					self.log("zfsGet", "Debug", str(properties))
				
				return properties
	
	def zfsList(self, command, specific = ""):
		if self.DEBUG_MODE:
			self.log("zfsList", "Information", "Running...")
		
		
		cmd = list(command)
		
		if specific <> "":
			cmd.append(specific)
		
		
		stdout, stderr, returncode = self.run(cmd)
		
		lines = stdout.split("\n")
		
		if len(lines) == 0:
			raise Exception("There doesn't appear to be anything returned.")
			
		else:
			if lines[0].startswith("no pools available"):
				raise Exception("No pools are available.")
				
			else:
				if self.DEBUG_MODE:
					for l in lines:
						self.log("zfsList", "Debug", l)
				
				
				datasets = []
				dataset = None
				
				num = 0
				
				
				# Clean up the data first
				cleaned = []
				
				for l in lines:
					l = self.trimExcessiveSpaces(l).replace("\t", "").strip()
					cleaned.append(l)
				
				lines = cleaned
				cleaned = None
				
				
				for l in lines:
					if not l.startswith("NAME USED AVAIL REFER MOUNTPOINT"):
						d = l.split(" ")
						
						if len(d) == 5:
							dataset = {"name": d[0], "used": d[1], "available": d[2], "refer": d[3], "mountpoint": d[4]}
							datasets.append(dataset)
				
				
				if self.DEBUG_MODE:
					self.log("zfsList", "Debug", str(datasets))
				
				return datasets
	
	def zfsListEverything(self, specific = ""):
		ret = []
		
		for fs in self.zfsList(self.ZFS_LIST_FILESYSTEMS, specific):
			ret.append(fs)
		
		for ss in self.zfsList(self.ZFS_LIST_SNAPSHOTS):
			ret.append(ss)
		
		for vol in self.zfsList(self.ZFS_LIST_VOLUMES):
			ret.append(vol)
		
		return ret
	
	def zfsListFilesystems(self, specific = ""):
		return self.zfsList(self.ZFS_LIST_FILESYSTEMS, specific)
	
	def zfsListSnapshots(self):
		return self.zfsList(self.ZFS_LIST_SNAPSHOTS)
	
	def zfsListVolumes(self):
		return self.zfsList(self.ZFS_LIST_VOLUMES)
	
	def zfsPoolIOStat(self, specific = ""):
		if self.DEBUG_MODE:
			self.log("zfsPoolIOStat", "Information", "Running...")
		
		
		cmd = list(self.ZPOOL_IOSTAT)
		
		if specific <> "":
			cmd.append(specific)
		
		
		stdout, stderr, returncode = self.run(cmd)
		
		lines = stdout.split("\n")
		
		if len(lines) == 0:
			raise Exception("There doesn't appear to be anything returned.")
			
		else:
			if lines[0].startswith("no pools available"):
				raise Exception("No pools are available.")
				
			else:
				if self.DEBUG_MODE:
					for l in lines:
						self.log("zfsPoolIOStat", "Debug", l)
				
				
				pools = []
				pool = None
				
				num = 0
				
				
				# Clean up the data first
				cleaned = []
				
				for l in lines:
					l = self.trimExcessiveSpaces(l).replace("\t", "").strip()
					cleaned.append(l)
				
				lines = cleaned
				cleaned = None
				
				
				for l in lines:
					if not l.startswith("pool alloc free read write read write") and not l.startswith("-"):
						d = l.split(" ")
						
						if len(d) == 7:
							pool = {"pool": d[0], "capacity": {"allocated": d[1], "free": d[2]}, "operations": {"read": d[3], "write": d[4]}, "bandwidth": {"read": d[5], "write": d[6]}}
							pools.append(pool)
				
				
				if self.DEBUG_MODE:
					self.log("zfsPoolIOStat", "Debug", str(pools))
				
				return pools
	
	def zfsPoolList(self, specific = ""):
		if self.DEBUG_MODE:
			self.log("zfsPoolList", "Information", "Running...")
		
		
		cmd = list(self.ZPOOL_LIST)
		
		if specific <> "":
			cmd.append(specific)
		
		
		stdout, stderr, returncode = self.run(cmd)
		
		lines = stdout.split("\n")
		
		if len(lines) == 0:
			raise Exception("There doesn't appear to be anything returned.")
			
		else:
			if lines[0].startswith("no pools available"):
				raise Exception("No pools are available.")
				
			else:
				if self.DEBUG_MODE:
					for l in lines:
						self.log("zfsPoolList", "Debug", l)
				
				
				pools = []
				pool = None
				
				num = 0
				
				
				# Clean up the data first
				cleaned = []
				
				for l in lines:
					l = self.trimExcessiveSpaces(l).replace("\t", "").strip()
					cleaned.append(l)
				
				lines = cleaned
				cleaned = None
				
				
				for l in lines:
					if not l.startswith("NAME SIZE ALLOC FREE CAP DEDUP HEALTH ALTROOT"):
						d = l.split(" ")
						
						if len(d) == 8:
							pool = {"name": d[0], "size": d[1], "allocated": d[2], "free": d[3], "capacity": d[4], "deduplication": d[5], "health": d[6], "alternativeroot": d[7]}
							pools.append(pool)
				
				
				if self.DEBUG_MODE:
					self.log("zfsPoolList", "Debug", str(pools))
				
				return pools
	
	def zfsPoolStatus(self, specific = ""):
		if self.DEBUG_MODE:
			self.log("zfsPoolStatus", "Information", "Running...")
		
		
		cmd = list(self.ZPOOL_STATUS)
		
		if specific <> "":
			cmd.append(specific)
		
		
		stdout, stderr, returncode = self.run(cmd)
		
		lines = stdout.split("\n")
		
		if len(lines) == 0:
			raise Exception("There doesn't appear to be anything returned.")
			
		else:
			if lines[0].startswith("no pools available"):
				raise Exception("No pools are available.")
				
			else:
				if self.DEBUG_MODE:
					for l in lines:
						self.log("zfsPoolStatus", "Debug", l)
				
				
				pools = []
				pool = None
				
				num = 0
				
				
				# Clean up the data first
				cleaned = []
				
				for l in lines:
					l = self.trimExcessiveSpaces(l).replace("\t", "").strip()
					cleaned.append(l)
				
				lines = cleaned
				cleaned = None
				
				
				for l in lines:
					# Keyword searching
					if l.find(self.ZPOOL_KEYWORD_POOL) <> -1:
						# New pool
						if self.DEBUG_MODE:
							self.log("zfsPoolStatus", "Information", "Pool found.")
						
						pool = {"pool": "", "state": "", "status": "", "action": "", "see": "", "scan": "", "config": {"type": "", "disks": []}, "errors": ""}
					
					for k in self.ZPOOL_KEYWORDS:
						if l.find(k) <> -1:
							self.log("zfsPoolStatus", "Debug", "%s keyword found." % k)
							
							
							k = k.replace(":", "")
							val = l.split(": ")
							
							if len(val) == 2:
								pool[k] = val[1]
								
								
								# Status runs on multiple lines
								if k == "status":
									num3 = 1
									done = False
									line_once = False
									
									while True:
										m = lines[num + num3]
										
										for kk in self.ZPOOL_KEYWORDS:
											if m.find(kk) == -1:
												if not line_once:
													pool[k] += " "
													pool[k] += m
													
													line_once = True
												
											else:
												done = True
												break
										
										
										if done:
											break
										
										line_once = False
										num3 += 1
					
					
					# Grab the type of pool
					if l.startswith("NAME STATE READ WRITE CKSUM"):
						pool["config"]["type"] = lines[num + self.ZPOOL_DISK_LINE_OFFSET].split(" ")[0].split("-")[0]
						
						# Grab the disks
						disks = []
						disk = None
						num2 = 1
						
						while True:
							m = lines[num + num2 + self.ZPOOL_DISK_LINE_OFFSET]
							
							if m.strip() == "":
								break
								
							else:
								d = m.split(" ")
								
								if len(d) == 5:
									# Disk appears normal
									disk = {"device": d[0], "state": d[1], "read": d[2], "write": d[3], "checksum": d[4]}
									disks.append(disk)
									
								elif len(d) == 7:
									# Disk likely to be offline
									disk = {"device": d[6], "state": d[1], "read": d[2], "write": d[3], "checksum": d[4]}
									disks.append(disk)
							
							num2 += 1
						
						pool["config"]["disks"] = disks
					
					
					if l.find(self.ZPOOL_KEYWORD_ERRORS) <> -1:
						pools.append(pool)
					
					self.log("zfsPoolStatus", "Information", repr(l))
					
					
					num += 1
				
				
				if self.DEBUG_MODE:
					self.log("zfsPoolStatus", "Debug", str(pools))
				
				return pools
